#!/usr/bin/env perl

use warnings 'FATAL', 'all';

use strict;
# Require strict checking of variable references, etc.

use utf8;
# Make Perl interpret the script as UTF-8.

use HTML::Entities;

#######################################################

my $BASENAME = 'ett-eng-fra-EL';
# Identify the filename base.

my $VERSION = 0;
# Identify the input file's version.

#######################################################

open my $in, '<:encoding(utf8)', "$BASENAME-$VERSION.txt" or die $!;
# Open the input file for reading.

open my $out, '>:encoding(utf8)', ("$BASENAME-" . ($VERSION + 1) . '.txt') or die $!;
# Create or truncate the output file and open it for writing.

my @dm;
# declare an array to store domain (semantic field) specifiers.

while (<$in>) {
    s/\r\n$//;
    # this file has DOS linebreaks (CRLF); this is how to remove them with a regex.
    
    $_ = decode_entities($_);
    # decode any HTML entities: &eacute; etc.
    
    next unless $_ ne '';
    # ignore the line if it's empty.
    
    if (m{<h3>(.+)</h3>}) {
    # if the line matches:
        
        @dm = split / - /, $1;
        # split the captured text into a list of domains.
                
        $dm[1] ||= $dm[0];
        # make sure the second (French) is present at least as an empty string.
        # otherwise, the corresponding column will not be created in the output
        # for the cases where there is no specified French domain, and the
        # serialization scripts will not work.

        $dm[1] = fra_ex($dm[1]);
        # extract the correct expression for the French domain (see function below).
        
        $dm[1] =~ s/⫷wc:noun⫸//;
        
        next;
        # continue immediately to the next line.
    }
    
    my ($art, $ett, $eng, $fra);
    # declare variables to store text from art-269 (integers), ett-000, eng-000, and fra-000.
    
    if (/^(\d+) +(.+)$/) {
    # if this entry is a number in ett (Etruscan):
        
        $eng = $fra = '';

        ($art, $ett) = ($1, $2);
        # store the first capture as the integer expression, and the second as ett.
        
        $ett =~ s/ - /‣/g;
        # replace the matched pattern with the standard synonym delimiter.
    }
    else {
        ($ett, $eng, $fra) = split / - /;
        # split on the indicated delimiter, storing the result as expressions in
        # ett, eng, and fra.
        
        $art = '';
        $eng ||= '';
        $fra ||= '';
        
        $ett =~ s/, +/‣/g;
        # replace the matched pattern with the standard synonym delimiter.
        
        $ett =~ s/^(.*)\((.+?)\)(.*)$/$1$3‣$1$2$3/g;
        # expand parenthesized items into two synonymous expressions.
        # 'pulum(chva)' becomes 'pulum‣pulumchva', etc.

        print "$_\n" if !defined $eng || !defined $fra;

        foreach my $txt ($eng, $fra) {
        # for each English and French expression:
            
            $txt =~ s/, */‣/g;
            # replace the matched pattern with the standard synonym delimiter.            
            
            $txt =~ s| / |‣|g;
            # replace the matched pattern with the standard synonym delimiter.

            $txt =~ s/\(s\)$//;
            # remove parenthesized plural '(s)'.
        }
        
        $fra = fra_ex($fra);
        # extract the correct French expression (see function below).
    }
    
    print $out join("\t", $ett, $eng, $fra, $art, @dm), "\n";
}

close $in;
# Close the input file.

close $out;
# Close the output file.

# function to transform a French entry into its PanLex expression.
sub fra_ex {
    my ($fra) = @_;
    # read the first argument into $fra.
    
    $fra =~ s/\bl'/⫷wc:noun⫸/g;
    # remove elided articles.
    
    $fra =~ s/\b(le|la|les) +/⫷wc:noun⫸/g;
    # remove full articles.
    
    $fra =~ s/ +de\b//g;
    # remove 'de'.
    
    return $fra;
    # return the result.
}
